/**
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/
 */
// Deliberate syntax error, should generate a worker exception!
for (var index = 0; index < 100) {}
