
/*
Copyright (c) 2019 The Khronos Group Inc.
Use of this source code is governed by an MIT-style license that can be
found in the LICENSE.txt file.
*/


#ifdef GL_ES
precision mediump float;
#endif
void main()
{
    bool b;
    if (b)
    {
        int i = 1;
        i++;
    }
    i++;  // i is not declared in this scope
}
