
/*
Copyright (c) 2019 The Khronos Group Inc.
Use of this source code is governed by an MIT-style license that can be
found in the LICENSE.txt file.
*/


void main()
{
    int a  // semicolon missing at the end of the statement
    gl_Position = vec4(a);
}
