/**
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/
 */
"use strict";

var b = new Blob(["123"], { type: "foo/bar" });
console.log({ msg: "consoleAndBlobs", blob: b });
